@@ -29,14 +29,13 @@
 #include "compat.h"
 #include "configfile.h"
 #include "strfctns.h"
-#include "util.h"
+#include "xalloc.h"
 
 extern char		*__progname;
 
 static EZCONFIG		 ezConfig;
 static const char	*blankString = "";
 
-void	        freeConfig(EZCONFIG *);
 unsigned int	checkDecoderLine(const char *, const char *, long);
 unsigned int	checkEncoderLine(const char *, const char *, long);
 unsigned int	checkFormatLine(const char *, const char *, long);
