@@ -132,7 +132,11 @@ void		usageHelp(void);
 int		ez_shutdown(int);
 
 #ifdef HAVE_SIGNALS
-void	sig_handler(int);
+void		sig_handler(int);
+
+# ifndef SIG_IGN
+#  define SIG_IGN	 (void (*)(int))1
+# endif /* !SIG_IGN */
 
 void
 sig_handler(int sig)
@@ -559,6 +563,11 @@ openResource(shout_t *shout, const char *fileName, int *popenFlag,
 	char		*pCommandString = NULL;
 	metadata_t	*mdata;
 
+	if (mdata_p != NULL)
+		*mdata_p = NULL;
+	if (songLen != NULL)
+		*songLen = 0;
+
 	if (strcmp(fileName, "stdin") == 0) {
 		if (metadataFromProgram) {
 			if ((mdata = getMetadata(pezConfig->metadataProgram)) == NULL)
@@ -573,9 +582,6 @@ openResource(shout_t *shout, const char *fileName, int *popenFlag,
 				metadata_free(&mdata);
 		}
 
-		if (vFlag)
-			printf("%s: Reading from standard input\n",
-			       __progname);
 		if (isStdin != NULL)
 			*isStdin = 1;
 #ifdef WIN32
@@ -852,7 +858,10 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 		if (errno == EINTR) {
 			clearerr(filepstream);
 			ret = STREAM_CONT;
-		} else
+		} else if (errno == EBADF && isStdin)
+			printf("%s: No (more) data available on standard input\n",
+			       __progname);
+		else
 			printf("%s: sendStream(): Error while reading '%s': %s\n",
 			       __progname, fileName, strerror(errno));
 	}
@@ -881,7 +890,8 @@ streamFile(shout_t *shout, const char *fileName)
 		char	*tmp, *metaData;
 
 		tmp = metadata_assemble_string(mdata);
-		metaData = UTF8toCHAR(tmp, ICONV_REPLACE);
+		if ((metaData = UTF8toCHAR(tmp, ICONV_REPLACE)) == NULL)
+			metaData = xstrdup("(unknown title)");
 		xfree(tmp);
 		printf("%s: Streaming ``%s''", __progname, metaData);
 		if (vFlag)
@@ -895,9 +905,10 @@ streamFile(shout_t *shout, const char *fileName)
 			setMetadata(shout, mdata, NULL);
 
 		metadata_free(&mdata);
-	}
+	} else if (isStdin)
+		printf("%s: Streaming from standard input\n", __progname);
 
-	if (songLen >= 0)
+	if (songLen > 0)
 		songLenStr = xstrdup(getTimeString(songLen));
 	ez_gettimeofday((void *)&startTime);
 	do {
@@ -1243,6 +1254,16 @@ main(int argc, char *argv[])
 			return (ez_shutdown(1));
 		}
 	}
+	/*
+	 * Ignore SIGPIPE, which has been seen to give a long-running ezstream
+	 * process trouble. EOF and/or EPIPE are also easier to handle.
+	 */
+	act.sa_handler = SIG_IGN;
+	if (sigaction(SIGPIPE, &act, NULL) == -1) {
+		printf("%s: sigaction(): %s\n",
+		       __progname, strerror(errno));
+		return (ez_shutdown(1));
+	}
 #endif /* HAVE_SIGNALS */
 
 	if (shout_open(shout) == SHOUTERR_SUCCESS) {
