@@ -37,11 +37,13 @@
 # include <taglib/tag_c.h>
 #endif
 #include <vorbis/vorbisfile.h>
+#include <shout/shout.h>
 
 #include "compat.h"
 #include "metadata.h"
 #include "strfctns.h"
 #include "util.h"
+#include "xalloc.h"
 
 extern char		*__progname;
 extern int		 vFlag;
@@ -54,6 +56,7 @@ struct metadata {
 	char	*artist;
 	char	*title;
 	int	 songLen;
+	int	 normalize;
 	int	 program;
 };
 
@@ -62,7 +65,7 @@ struct ID3Tag {
 	char trackName[30];
 	char artistName[30];
 	char albumName[30];
-	char year[3];
+	char year[4];
 	char comment[30];
 	char genre;
 };
@@ -74,6 +77,7 @@ void    	metadata_clean_md(metadata_t *);
 void		metadata_get_extension(char *, size_t, const char *);
 char *		metadata_get_name(const char *);
 void		metadata_process_md(metadata_t *);
+void		metadata_normalize_string(char **);
 
 metadata_t *
 metadata_create(const char *filename)
@@ -107,10 +111,16 @@ metadata_use_taglib(metadata_t *md, FILE **filep)
 
 	metadata_clean_md(md);
 	taglib_set_string_management_enabled(0);
+#ifdef HAVE_ICONV
+	taglib_set_strings_unicode(1);
+#else
 	taglib_set_strings_unicode(0);
+#endif /* HAVE_ICONV */
 
-	if (md->string != NULL)
+	if (md->string != NULL) {
 		xfree(md->string);
+		md->string = NULL;
+	}
 
 	if ((tf = taglib_file_new(md->filename)) == NULL) {
 		md->string = metadata_get_name(md->filename);
@@ -124,14 +134,14 @@ metadata_use_taglib(metadata_t *md, FILE **filep)
 	if (str != NULL) {
 		if (strlen(str) > 0)
 			md->artist = xstrdup(str);
-		xfree(str);
+		free(str);
 	}
 
 	str = taglib_tag_title(tt);
 	if (str != NULL) {
 		if (strlen(str) > 0)
 			md->title = xstrdup(str);
-		xfree(str);
+		free(str);
 	}
 
 	md->songLen = taglib_audioproperties_length(ta);
@@ -172,12 +182,12 @@ metadata_use_self(metadata_t *md, FILE **filep)
 	if (strcmp(extension, ".mp3") == 0) {
 		memset(&id3tag, 0, sizeof(id3tag));
 		fseek(*filep, -128L, SEEK_END);
-		fread(&id3tag, 1, 127, *filep);
-		if (strncmp(id3tag.tag, "TAG", strlen("TAG")) == 0) {
+		fread(&id3tag, 1, sizeof(struct ID3Tag), *filep);
+		if (memcmp(id3tag.tag, "TAG", 3) == 0) {
 			if (strlen(id3tag.artistName) > 0)
-				md->artist = xstrdup(id3tag.artistName);
+				md->artist = CHARtoUTF8(id3tag.artistName, ICONV_REPLACE);
 			if (strlen(id3tag.trackName) > 0)
-				md->title = xstrdup(id3tag.trackName);
+				md->title = CHARtoUTF8(id3tag.trackName, ICONV_REPLACE);
 		}
 	} else if (strcmp(extension, ".ogg") == 0) {
 		OggVorbis_File	vf;
@@ -248,12 +258,18 @@ metadata_clean_md(metadata_t *md)
 		abort();
 	}
 
-	if (md->string != NULL)
+	if (md->string != NULL) {
 		xfree(md->string);
-	if (md->artist != NULL)
+		md->string = NULL;
+	}
+	if (md->artist != NULL) {
 		xfree(md->artist);
-	if (md->title != NULL)
+		md->artist = NULL;
+	}
+	if (md->title != NULL) {
 		xfree(md->title);
+		md->title = NULL;
+	}
 }
 
 void
@@ -316,10 +332,47 @@ metadata_process_md(metadata_t *md)
 
 	if (md->string == NULL)
 		md->string = metadata_assemble_string(md);
+
+	if (md->normalize) {
+		metadata_normalize_string(&md->string);
+		metadata_normalize_string(&md->artist);
+		metadata_normalize_string(&md->title);
+	}
+}
+
+void
+metadata_normalize_string(char **s)
+{
+	char	*str, *cp, *tmpstr, *tp;
+	int	 is_space;
+
+	if (s == NULL || (str = *s) == NULL || strlen(str) == 0)
+		return;
+
+	tmpstr = xcalloc(strlen(str) + 1, sizeof(char));
+
+	tp = tmpstr;
+	is_space = 1;
+	for (cp = str; *cp != '\0'; cp++) {
+		if (*cp == ' ') {
+			if (!is_space && strlen(tmpstr) > 0 &&
+			    tmpstr[strlen(tmpstr) - 1] != ' ')
+				*tp++ = ' ';
+			is_space = 1;
+		} else {
+			*tp++ = *cp;
+			is_space = 0;
+		}
+	}
+	if (strlen(tmpstr) > 0 && tmpstr[strlen(tmpstr) - 1] == ' ')
+		tmpstr[strlen(tmpstr) - 1] = '\0';
+
+	xfree(str);
+	*s = xrealloc(tmpstr, strlen(tmpstr) + 1, sizeof (char));
 }
 
 metadata_t *
-metadata_file(const char *filename)
+metadata_file(const char *filename, int normalize)
 {
 	metadata_t	*md;
 
@@ -335,11 +388,13 @@ metadata_file(const char *filename)
 		return (NULL);
 	}
 
+	md->normalize = normalize;
+
 	return (md);
 }
 
 metadata_t *
-metadata_program(const char *program)
+metadata_program(const char *program, int normalize)
 {
 	metadata_t	*md;
 #ifdef HAVE_STAT
@@ -384,23 +439,26 @@ metadata_program(const char *program)
 	fclose(filep);
 #endif /* HAVE_STAT */
 
+	md->normalize = normalize;
+
 	return (md);
 }
 
 void
-metadata_free(metadata_t **md)
+metadata_free(metadata_t **md_p)
 {
-	metadata_t	*tmp;
+	metadata_t	*md;
 
-	if (md == NULL || *md == NULL)
+	if (md_p == NULL || (md = *md_p) == NULL)
 		return;
 
-	tmp = *md;
-
-	if (tmp->filename != NULL)
-		xfree(tmp->filename);
-	metadata_clean_md(tmp);
-	xfree(*md);
+	if (md->filename != NULL) {
+		xfree(md->filename);
+		md->filename = NULL;
+	}
+	metadata_clean_md(md);
+	xfree(*md_p);
+	*md_p = NULL;
 }
 
 
@@ -467,18 +525,24 @@ metadata_program_update(metadata_t *md, enum metadata_request md_req)
 			return (1);
 	case METADATA_STRING:
 		strlcpy(command, md->filename, sizeof(command));
-		if (md->string != NULL)
+		if (md->string != NULL) {
 			xfree(md->string);
+			md->string = NULL;
+		}
 		break;
 	case METADATA_ARTIST:
 		snprintf(command, sizeof(command), "%s artist", md->filename);
-		if (md->artist != NULL)
+		if (md->artist != NULL) {
 			xfree(md->artist);
+			md->artist = NULL;
+		}
 		break;
 	case METADATA_TITLE:
 		snprintf(command, sizeof(command), "%s title", md->filename);
-		if (md->title != NULL)
+		if (md->title != NULL) {
 			xfree(md->title);
+			md->title = NULL;
+		}
 		break;
 	default:
 		printf("%s: metadata_program_update(): Internal error: Unknown md_req\n",
@@ -518,10 +582,8 @@ metadata_program_update(metadata_t *md, enum metadata_request md_req)
 		printf("%s: Warning: Metadata string received via '%s' is too long and has been truncated\n",
 		       __progname, command);
 
-	if (buf[0] != '\0' && buf[strlen(buf) - 1] == '\n')
-		buf[strlen(buf) - 1] = '\0';
-	if (buf[0] != '\0' && buf[strlen(buf) - 1] == '\r')
-		buf[strlen(buf) - 1] = '\0';
+	buf[strcspn(buf, "\n")] = '\0';
+	buf[strcspn(buf, "\r")] = '\0';
 
 	switch (md_req) {
 	case METADATA_STRING:
@@ -547,6 +609,12 @@ metadata_program_update(metadata_t *md, enum metadata_request md_req)
 		abort();
 	}
 
+	if (md->normalize) {
+		metadata_normalize_string(&md->string);
+		metadata_normalize_string(&md->artist);
+		metadata_normalize_string(&md->title);
+	}
+
 	return (1);
 }
 
@@ -629,7 +697,7 @@ metadata_get_length(metadata_t *md)
 char *
 metadata_assemble_string(metadata_t *md)
 {
-	size_t	  siz;
+	size_t	  len;
 	char	 *str;
 
 	if (md == NULL) {
@@ -641,23 +709,23 @@ metadata_assemble_string(metadata_t *md)
 	if (md->artist == NULL && md->title == NULL && md->program == 0)
 		return (metadata_get_name(md->filename));
 
-	siz = 0;
+	len = 0;
 	if (md->artist != NULL)
-		siz += strlen(md->artist);
+		len += strlen(md->artist);
 	if (md->title != NULL) {
-		if (siz > 0)
-			siz += strlen(" - ");
-		siz += strlen(md->title);
+		if (len > 0)
+			len += strlen(" - ");
+		len += strlen(md->title);
 	}
-	siz++;
-	str = xcalloc(1, siz);
+	len++;
+	str = xcalloc(len, sizeof(char));
 
 	if (md->artist != NULL)
-		strlcpy(str, md->artist, siz);
+		strlcpy(str, md->artist, len);
 	if (md->title != NULL) {
 		if (md->artist != NULL)
-			strlcat(str, " - ", siz);
-		strlcat(str, md->title, siz);
+			strlcat(str, " - ", len);
+		strlcat(str, md->title, len);
 	}
 
 	return (str);
